@@ -75,20 +75,16 @@ namespace DigikamAdjustCurvesImagesPlugin
 {
 
 AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint width, uint height)
-                 : KDialogBase(Plain, i18n("Adjust Color Curves"), Help|User1|Ok|Cancel, Ok,
-                               parent, 0, true, true, i18n("&Reset Values"))
+                 : ImageTabDialog(parent, i18n("Adjust Color Curves"), "adjustcurves", 
+                                  true, true, false)
 {
-    parentWidget()->setCursor( KCursor::waitCursor() );
-    
     // Create an empty instance of curves to use.
     m_curves = new Digikam::ImageCurves();
 
-    setButtonWhatsThis ( User1, i18n("<p>Reset curves' values from the current selected channel.") );
-
     // About data and help button.
 
     KAboutData* about = new KAboutData("digikamimageplugins",
-                                       I18N_NOOP("Adjust Curves"),
+                                       I18N_NOOP("Adjust Color Curves"),
                                        digikamimageplugins_version,
                                        I18N_NOOP("An image-histogram-curves adjustment plugin for digiKam."),
                                        KAboutData::License_GPL,
@@ -99,49 +95,16 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
     about->addAuthor("Gilles Caulier", I18N_NOOP("Author and maintainer"),
                      "caulier dot gilles at free.fr");
 
-    m_helpButton = actionButton( Help );
-    KHelpMenu* helpMenu = new KHelpMenu(this, about, false);
-    helpMenu->menu()->removeItemAt(0);
-    helpMenu->menu()->insertItem(i18n("Adjust Curves Handbook"), this, SLOT(slotHelp()), 0, -1, 0);
-    m_helpButton->setPopup( helpMenu->menu() );
-
+    setAboutData(about);
+    
     // -------------------------------------------------------------
 
-    QGridLayout* topLayout = new QGridLayout( plainPage(), 3, 2 , marginHint(), spacingHint());
+    QWidget *gboxSettings = new QWidget(plainPage());
+    QGridLayout* grid = new QGridLayout( gboxSettings, 7, 5, marginHint(), spacingHint());
 
-    QFrame *headerFrame = new QFrame( plainPage() );
-    headerFrame->setFrameStyle(QFrame::Panel|QFrame::Sunken);
-    QHBoxLayout* layout = new QHBoxLayout( headerFrame );
-    layout->setMargin( 2 ); // to make sure the frame gets displayed
-    layout->setSpacing( 0 );
-    QLabel *pixmapLabelLeft = new QLabel( headerFrame, "pixmapLabelLeft" );
-    pixmapLabelLeft->setScaledContents( false );
-    layout->addWidget( pixmapLabelLeft );
-    QLabel *labelTitle = new QLabel( i18n("Adjust Color Curves"), headerFrame, "labelTitle" );
-    layout->addWidget( labelTitle );
-    layout->setStretchFactor( labelTitle, 1 );
-    topLayout->addMultiCellWidget(headerFrame, 0, 0, 0, 1);
-
-    QString directory;
-    KGlobal::dirs()->addResourceType("digikamimageplugins_banner_left", KGlobal::dirs()->kde_default("data") +
-                                                                        "digikamimageplugins/data");
-    directory = KGlobal::dirs()->findResourceDir("digikamimageplugins_banner_left",
-                                                 "digikamimageplugins_banner_left.png");
-
-    pixmapLabelLeft->setPaletteBackgroundColor( QColor(201, 208, 255) );
-    pixmapLabelLeft->setPixmap( QPixmap( directory + "digikamimageplugins_banner_left.png" ) );
-    labelTitle->setPaletteBackgroundColor( QColor(201, 208, 255) );
-
-    // -------------------------------------------------------------
-
-    QGroupBox *gbox = new QGroupBox(plainPage());
-    gbox->setFlat(false);
-    gbox->setTitle(i18n("Settings"));
-    QGridLayout* grid = new QGridLayout( gbox, 4, 6, 20, spacingHint());
-
-    QLabel *label1 = new QLabel(i18n("Channel:"), gbox);
+    QLabel *label1 = new QLabel(i18n("Channel:"), gboxSettings);
     label1->setAlignment ( Qt::AlignRight | Qt::AlignVCenter );
-    m_channelCB = new QComboBox( false, gbox );
+    m_channelCB = new QComboBox( false, gboxSettings );
     m_channelCB->insertItem( i18n("Luminosity") );
     m_channelCB->insertItem( i18n("Red") );
     m_channelCB->insertItem( i18n("Green") );
@@ -157,26 +120,42 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
                                        "This channel corresponds to the transparency value and "
                                        "is supported by some image formats, such as PNG or GIF."));
 
-    QLabel *label2 = new QLabel(i18n("Scale:"), gbox);
-    label2->setAlignment ( Qt::AlignRight | Qt::AlignVCenter);
-    m_scaleCB = new QComboBox( false, gbox );
-    m_scaleCB->insertItem( i18n("Linear") );
-    m_scaleCB->insertItem( i18n("Logarithmic") );
-    m_scaleCB->setCurrentText( i18n("Logarithmic") );
-    QWhatsThis::add( m_scaleCB, i18n("<p>Select here the histogram scale.<p>"
+    m_scaleBG = new QHButtonGroup(gboxSettings);
+    m_scaleBG->setExclusive(true);
+    m_scaleBG->setFrameShape(QFrame::NoFrame);
+    m_scaleBG->setInsideMargin( 0 );
+    QWhatsThis::add( m_scaleBG, i18n("<p>Select here the histogram scale.<p>"
                                      "If the image's maximal counts are small, you can use the linear scale.<p>"
                                      "Logarithmic scale can be used when the maximal counts are big; "
-                                     "if it is used, all values (small and large) will be visible on the "
-                                     "graph."));
+                                     "if it is used, all values (small and large) will be visible on the graph."));
+    
+    QPushButton *linHistoButton = new QPushButton( m_scaleBG );
+    QToolTip::add( linHistoButton, i18n( "<p>Linear" ) );
+    m_scaleBG->insert(linHistoButton, Digikam::CurvesWidget::LinScaleHistogram);
+    KGlobal::dirs()->addResourceType("histogram-lin", KGlobal::dirs()->kde_default("data") + "digikam/data");
+    QString directory = KGlobal::dirs()->findResourceDir("histogram-lin", "histogram-lin.png");
+    linHistoButton->setPixmap( QPixmap( directory + "histogram-lin.png" ) );
+    linHistoButton->setToggleButton(true);
+    
+    QPushButton *logHistoButton = new QPushButton( m_scaleBG );
+    QToolTip::add( logHistoButton, i18n( "<p>Logarithmic" ) );
+    m_scaleBG->insert(logHistoButton, Digikam::CurvesWidget::LogScaleHistogram);
+    KGlobal::dirs()->addResourceType("histogram-log", KGlobal::dirs()->kde_default("data") + "digikam/data");
+    directory = KGlobal::dirs()->findResourceDir("histogram-log", "histogram-log.png");
+    logHistoButton->setPixmap( QPixmap( directory + "histogram-log.png" ) );
+    logHistoButton->setToggleButton(true);       
 
+    QHBoxLayout* l1 = new QHBoxLayout();
+    l1->addWidget(m_scaleBG);
+    l1->addStretch(10);
+    
     grid->addMultiCellWidget(label1, 0, 0, 1, 1);
     grid->addMultiCellWidget(m_channelCB, 0, 0, 2, 2);
-    grid->addMultiCellWidget(label2, 0, 0, 4, 4);
-    grid->addMultiCellWidget(m_scaleCB, 0, 0, 5, 5);
+    grid->addMultiCellLayout(l1, 0, 0, 4, 5);
     
-    QLabel *label5 = new QLabel(i18n("Type:"), gbox);
+    QLabel *label5 = new QLabel(i18n("Type:"), gboxSettings);
     label5->setAlignment ( Qt::AlignRight | Qt::AlignVCenter);
-    m_typeCB = new QComboBox( false, gbox );
+    m_typeCB = new QComboBox( false, gboxSettings );
     m_typeCB->insertItem( i18n("Smooth") );
     m_typeCB->insertItem( i18n("Free") );
     m_typeCB->setCurrentText( i18n("Smooth") );
@@ -184,44 +163,28 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
                                     "<b>Smooth</b>: this mode constrains the curve type to a smooth line with tension.<p>"
                                     "<b>Free</b>: with this mode, you can draw your curve free-hand with the mouse."));
 
-    m_labelPos = new QLabel(gbox);
-    m_labelPos->setAlignment ( Qt::AlignRight | Qt::AlignVCenter);
+    m_labelPos = new QLabel(gboxSettings);
                                      
     grid->addMultiCellWidget(label5, 1, 1, 1, 1);
     grid->addMultiCellWidget(m_typeCB, 1, 1, 2, 2);
-    grid->addMultiCellWidget(m_labelPos, 1, 1, 5, 5);
+    grid->addMultiCellWidget(m_labelPos, 2, 2, 1, 5);
     
-    QFrame *frame = new QFrame(gbox);
-    frame->setFrameStyle(QFrame::Panel|QFrame::Sunken);
-    QVBoxLayout* l = new QVBoxLayout(frame, 5, 0);
-
-    m_vGradient = new Digikam::ColorGradientWidget( KSelector::Vertical, 20, gbox );
+    m_vGradient = new Digikam::ColorGradientWidget( Digikam::ColorGradientWidget::Vertical, 10, gboxSettings );
     m_vGradient->setColors( QColor( "white" ), QColor( "black" ) );
-    grid->addMultiCellWidget(m_vGradient, 2, 2, 0, 0);
+    grid->addMultiCellWidget(m_vGradient, 3, 3, 0, 0);
 
-    m_curvesWidget = new Digikam::CurvesWidget(256, 256, imageData, width, height, m_curves, frame);
+    m_curvesWidget = new Digikam::CurvesWidget(256, 256, imageData, width, height, m_curves, gboxSettings);
     QWhatsThis::add( m_curvesWidget, i18n("<p>This is the curve drawing of the selected image "
                                           "histogram channel"));
-    l->addWidget(m_curvesWidget, 0);
-    grid->addMultiCellWidget(frame, 2, 2, 1, 5);
+    grid->addMultiCellWidget(m_curvesWidget, 3, 3, 1, 5);
     
-    m_hGradient = new Digikam::ColorGradientWidget( KSelector::Horizontal, 20, gbox );
+    m_hGradient = new Digikam::ColorGradientWidget( Digikam::ColorGradientWidget::Horizontal, 10, gboxSettings );
     m_hGradient->setColors( QColor( "black" ), QColor( "white" ) );
-    grid->addMultiCellWidget(m_hGradient, 3, 3, 1, 5);
+    grid->addMultiCellWidget(m_hGradient, 4, 4, 1, 5);
     
-    topLayout->addMultiCellWidget(gbox, 1, 1, 0, 0);
-
     // -------------------------------------------------------------
-
-    QHGroupBox *gbox3 = new QHGroupBox(i18n("All Channels' Curves"), plainPage());
-    m_loadButton = new QPushButton(i18n("&Load..."), gbox3);
-    QWhatsThis::add( m_loadButton, i18n("<p>Load curve settings from a Gimp curves text file."));
-    m_saveButton = new QPushButton(i18n("&Save As..."), gbox3);
-    QWhatsThis::add( m_saveButton, i18n("<p>Save curve settings to a Gimp curves text file."));
-    m_resetButton = new QPushButton(i18n("&Reset All"), gbox3);
-    QWhatsThis::add( m_resetButton, i18n("<p>Reset all channels' curve values."));
     
-    m_pickerColorButtonGroup = new QHButtonGroup(gbox3);
+    m_pickerColorButtonGroup = new QHButtonGroup(gboxSettings);
     m_pickBlack = new QPushButton(m_pickerColorButtonGroup);
     m_pickerColorButtonGroup->insert(m_pickBlack, BlackTonal);
     KGlobal::dirs()->addResourceType("color-picker-black", KGlobal::dirs()->kde_default("data") +
@@ -255,42 +218,43 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
     m_pickerColorButtonGroup->setExclusive(true);
     m_pickerColorButtonGroup->setFrameShape(QFrame::NoFrame);
 
-    topLayout->addMultiCellWidget(gbox3, 3, 3, 0, 0);
+    m_resetButton = new QPushButton(i18n("&Reset"), gboxSettings);
+    QWhatsThis::add( m_resetButton, i18n("<p>Reset curves' values from the current selected channel."));
+
+    QHBoxLayout* l3 = new QHBoxLayout();
+    l3->addWidget(m_pickerColorButtonGroup);
+    l3->addWidget(m_resetButton);
+    l3->addStretch(10);
+    
+    grid->addMultiCellLayout(l3, 5, 5, 1, 5);
+    
+    // -------------------------------------------------------------
+            
+    m_overExposureIndicatorBox = new QCheckBox(i18n("Over exposure indicator"), gboxSettings);
+    QWhatsThis::add( m_overExposureIndicatorBox, i18n("<p>If you enable this option, over-exposed pixels "
+                                                      "from the target image preview will be over-colored. "
+                                                      "This will not have an effect on the final rendering."));
+    grid->addMultiCellWidget(m_overExposureIndicatorBox, 6, 6, 1, 5);
+    grid->setRowStretch(7, 10);
+    
+    setUserAreaWidget(gboxSettings);
 
     // -------------------------------------------------------------
 
-    QVGroupBox *gbox4 = new QVGroupBox(i18n("Preview"), plainPage());
-
-    QFrame *frame2 = new QFrame(gbox4);
-    frame2->setFrameStyle(QFrame::Panel|QFrame::Sunken);
-    QVBoxLayout* l2  = new QVBoxLayout(frame2, 5, 0);
-    m_previewOriginalWidget = new Digikam::ImageGuideWidget(300, 200, frame2, true, 
-                                                            Digikam::ImageGuideWidget::PickColorMode);
-    QWhatsThis::add( m_previewOriginalWidget, i18n("<p>You can see here the original image. You can pick a color on the image using the color "
-                                                   "picker tools to select shadow, middle, and highlight tones to adjust the curves' points in the Red, "
+    m_previewOriginalWidget = previewOriginalWidget();
+    QWhatsThis::add( m_previewOriginalWidget, i18n("<p>You can see here the original image. You can pick "
+                                                   "a color on the image using the color "
+                                                   "picker tools to select shadow, middle, and highlight "
+                                                   "tones to adjust the curves' points in the Red, "
                                                    "Green, Blue, and Luminosity Channels."));
-    l2->addWidget(m_previewOriginalWidget, 0, Qt::AlignCenter);
-
-    QFrame *frame3 = new QFrame(gbox4);
-    frame3->setFrameStyle(QFrame::Panel|QFrame::Sunken);
-    QVBoxLayout* l3  = new QVBoxLayout(frame3, 5, 0);
-    m_previewTargetWidget = new Digikam::ImageWidget(300, 200, frame3);
-    QWhatsThis::add( m_previewTargetWidget, i18n("<p>You can see here the image's curve-adjustments preview."));
-    l3->addWidget(m_previewTargetWidget, 0, Qt::AlignCenter);
-
-    m_overExposureIndicatorBox = new QCheckBox(i18n("Over exposure indicator"), gbox4);
-    QWhatsThis::add( m_overExposureIndicatorBox, i18n("<p>If you enable this option, over-exposed pixels from the target image preview "
-                                                      "will be over-colored. This will not have an effect on the final rendering."));
-                                                          
-    topLayout->addMultiCellWidget(gbox4, 1, 3, 1, 1);
+    m_previewTargetWidget   = previewTargetWidget();
+    QWhatsThis::add( m_previewTargetWidget, i18n("<p>You can see here the image's "
+                                                 "curve-adjustments preview."));
 
     // -------------------------------------------------------------
     
-    adjustSize();
-    disableResize();
-
-    QTimer::singleShot(0, this, SLOT(slotResetAllChannels())); // Reset all parameters to the default values.
-    parentWidget()->setCursor( KCursor::arrowCursor()  );
+    // Reset all parameters to the default values.
+    QTimer::singleShot(0, this, SLOT(slotDefault()));
 
     // -------------------------------------------------------------
     
@@ -305,14 +269,17 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
             
     connect(m_overExposureIndicatorBox, SIGNAL(toggled (bool)),
             this, SLOT(slotEffect()));              
-    
+
+    connect(m_previewTargetWidget, SIGNAL(signalResized()),
+            this, SLOT(slotEffect()));                                                            
+            
     // -------------------------------------------------------------
     // ComboBox slots.
 
     connect(m_channelCB, SIGNAL(activated(int)),
             this, SLOT(slotChannelChanged(int)));
 
-    connect(m_scaleCB, SIGNAL(activated(int)),
+    connect(m_scaleBG, SIGNAL(released(int)),
             this, SLOT(slotScaleChanged(int)));
             
     connect(m_typeCB, SIGNAL(activated(int)),
@@ -322,29 +289,13 @@ AdjustCurveDialog::AdjustCurveDialog(QWidget* parent, uint *imageData, uint widt
     // Bouttons slots.
 
     connect(m_resetButton, SIGNAL(clicked()),
-            this, SLOT(slotResetAllChannels()));
-
-    connect(m_loadButton, SIGNAL(clicked()),
-            this, SLOT(slotLoadCurves()));
-
-    connect(m_saveButton, SIGNAL(clicked()),
-            this, SLOT(slotSaveCurves()));
+            this, SLOT(slotResetCurrentChannel()));
 }
 
 AdjustCurveDialog::~AdjustCurveDialog()
-{
-}
-
-void AdjustCurveDialog::slotHelp()
-{
-    KApplication::kApplication()->invokeHelp("adjustcurves", "digikamimageplugins");
-}
-
-void AdjustCurveDialog::closeEvent(QCloseEvent *e)
 {
     delete m_curvesWidget;
     delete m_curves;
-    e->accept();
 }
 
 void AdjustCurveDialog::slotSpotColorChanged(const QColor &color, bool release)
@@ -392,16 +343,7 @@ void AdjustCurveDialog::slotSpotColorChanged(const QColor &color, bool release)
     slotEffect();  
 }
 
-void AdjustCurveDialog::slotResetAllChannels()
-{
-    for (int channel = 0 ; channel < 5 ; channel++)
-       m_curves->curvesChannelReset(channel);
-
-    m_curvesWidget->reset();
-    slotEffect();
-}
-
-void AdjustCurveDialog::slotUser1()
+void AdjustCurveDialog::slotResetCurrentChannel()
 {
     m_curves->curvesChannelReset(m_channelCB->currentItem());
 
@@ -414,18 +356,16 @@ void AdjustCurveDialog::slotPositionChanged(int x, int y)
     if ( x == -1 && y == -1)
        m_labelPos->clear();
     else
-       m_labelPos->setText(i18n("x:%1   y:%2").arg(x).arg(y));
+       m_labelPos->setText(i18n("Position: x:%1  y:%2").arg(x).arg(y));
 }
 
 void AdjustCurveDialog::slotEffect()
 {
-    Digikam::ImageIface* ifaceOrg = m_previewOriginalWidget->imageIface();
-
     Digikam::ImageIface* ifaceDest = m_previewTargetWidget->imageIface();
 
-    uint* orgData = ifaceOrg->getPreviewData();
-    int   w       = ifaceOrg->previewWidth();
-    int   h       = ifaceOrg->previewHeight();
+    uint* orgData = ifaceDest->getPreviewData();
+    int   w       = ifaceDest->previewWidth();
+    int   h       = ifaceDest->previewHeight();
 
     // Create the new empty destination image data space.
     uint* desData = new uint[w*h];
@@ -437,7 +377,7 @@ void AdjustCurveDialog::slotEffect()
     m_curves->curvesLutProcess(orgData, desData, w, h);
 
     ifaceDest->putPreviewData(desData);
-    m_previewTargetWidget->update();
+    m_previewTargetWidget->updatePreview();
 
     delete [] orgData;
     delete [] desData;
@@ -445,13 +385,12 @@ void AdjustCurveDialog::slotEffect()
 
 void AdjustCurveDialog::slotOk()
 {
-    Digikam::ImageIface* ifaceOrg = m_previewOriginalWidget->imageIface();
+    kapp->setOverrideCursor( KCursor::waitCursor() );
+    Digikam::ImageIface ifaceDest(0, 0);
 
-    Digikam::ImageIface* ifaceDest = m_previewTargetWidget->imageIface();
-
-    uint* orgData = ifaceOrg->getOriginalData();
-    int   w       = ifaceOrg->originalWidth();
-    int   h       = ifaceOrg->originalHeight();
+    uint* orgData = ifaceDest.getOriginalData();
+    int   w       = ifaceDest.originalWidth();
+    int   h       = ifaceDest.originalHeight();
 
     // Create the new empty destination image data space.
     uint* desData = new uint[w*h];
@@ -462,7 +401,8 @@ void AdjustCurveDialog::slotOk()
     // Apply the lut to the image.
     m_curves->curvesLutProcess(orgData, desData, w, h);
 
-    ifaceDest->putOriginalData(i18n("Adjust Curve"), desData);
+    ifaceDest.putOriginalData(i18n("Adjust Curve"), desData);
+    kapp->restoreOverrideCursor();
 
     delete [] orgData;
     delete [] desData;
@@ -474,26 +414,31 @@ void AdjustCurveDialog::slotChannelChanged(int channel)
     switch(channel)
        {
        case LuminosityChannel:
+          m_hGradient->setColors( QColor( "black" ), QColor( "white" ) );
           m_curvesWidget->m_channelType = Digikam::CurvesWidget::ValueHistogram;
           m_vGradient->setColors( QColor( "white" ), QColor( "black" ) );
           break;
        
        case RedChannel:
+          m_hGradient->setColors( QColor( "black" ), QColor( "red" ) );
           m_curvesWidget->m_channelType = Digikam::CurvesWidget::RedChannelHistogram;
           m_vGradient->setColors( QColor( "red" ), QColor( "black" ) );
           break;
 
        case GreenChannel:
+          m_hGradient->setColors( QColor( "black" ), QColor( "green" ) );
           m_curvesWidget->m_channelType = Digikam::CurvesWidget::GreenChannelHistogram;
           m_vGradient->setColors( QColor( "green" ), QColor( "black" ) );
           break;
 
        case BlueChannel:
+          m_hGradient->setColors( QColor( "black" ), QColor( "blue" ) );
           m_curvesWidget->m_channelType = Digikam::CurvesWidget::BlueChannelHistogram;
           m_vGradient->setColors( QColor( "blue" ), QColor( "black" ) );
           break;
 
        case AlphaChannel:
+          m_hGradient->setColors( QColor( "black" ), QColor( "white" ) );
           m_curvesWidget->m_channelType = Digikam::CurvesWidget::AlphaChannelHistogram;
           m_vGradient->setColors( QColor( "white" ), QColor( "black" ) );
           break;
@@ -506,17 +451,7 @@ void AdjustCurveDialog::slotChannelChanged(int channel)
 
 void AdjustCurveDialog::slotScaleChanged(int scale)
 {
-    switch(scale)
-       {
-       case Linear:
-          m_curvesWidget->m_scaleType = Digikam::CurvesWidget::LinScaleHistogram;
-          break;
-       
-       case Logarithmic:
-          m_curvesWidget->m_scaleType = Digikam::CurvesWidget::LogScaleHistogram;
-          break;
-       }
-
+    m_curvesWidget->m_scaleType = scale;
     m_curvesWidget->repaint(false);
 }
 
@@ -542,7 +477,18 @@ void AdjustCurveDialog::slotCurveTypeChanged(int type)
     m_curvesWidget->curveTypeChanged();
 }
 
-void AdjustCurveDialog::slotLoadCurves()
+// Reset all settings.
+void AdjustCurveDialog::slotDefault()
+{
+    for (int channel = 0 ; channel < 5 ; channel++)
+       m_curves->curvesChannelReset(channel);
+
+    m_curvesWidget->reset();
+    slotEffect();
+}
+
+// Load all settings.
+void AdjustCurveDialog::slotUser3()
 {
     KURL loadCurvesFile;
 
@@ -563,7 +509,8 @@ void AdjustCurveDialog::slotLoadCurves()
     slotEffect();
 }
 
-void AdjustCurveDialog::slotSaveCurves()
+// Save all settings.
+void AdjustCurveDialog::slotUser2()
 {
     KURL saveCurvesFile;
 
